import {
  ClassType,
  PropertyType,
} from '@loafer/core/constants/types';

import {
  PouchScope,
} from '@loafer/pouches/constants/types';

interface PouchDefinition {

	/**
	 * The name of the parent definition of this pouch definition, if any.
	 */
  ParentName: PropertyType;
	/**
	 * The current pouch class name of this pouch definition.
	 * <p>Note that this does not have to be the actual class name used at runtime, in
	 * case of a child definition overriding/inheriting the class name from its parent.
	 * Also, this may just be the class that a factory method is called on, or it may
	 * even be empty in case of a factory pouch reference that a method is called on.
	 * Hence, do <i>not</i> consider this to be the definitive pouch type at runtime but
	 * rather only use it for parsing purposes at the individual pouch definition level.
	 * @see #getParentName()
	 * @see #getFactoryPouchName()
	 * @see #getFactoryMethodName()
	 */
  PouchClassName: string;
	/**
	 * Override the target scope of this pouch, specifying a new scope name.
	 * @see #SCOPE_SINGLETON
	 * @see #SCOPE_PROTOTYPE
	 */
  Scope: string;
	/**
	 * Whether this pouch should be lazily initialized.
	 * <p>If {@code false}, the pouch will get instantiated on startup by pouch
	 * factories that perform eager initialization of singletons.
	 */
  LazyInit: boolean;
	/**
	 * The names of the pouchs that this pouch depends on being initialized.
	 * The pouch factory will guarantee that these pouchs get initialized first.
	 */
  DependsOn: PropertyType[];
	/**
	 * Whether this pouch is a candidate for getting autowired into some other pouch.
	 * <p>Note that this flag is designed to only affect type-based autowiring.
	 * It does not affect explicit references by name, which will get resolved even
	 * if the specified pouch is not marked as an autowire candidate. As a consequence,
	 * autowiring by name will nevertheless inject a pouch if the name matches.
	 */
  AutowireCandidate: boolean;
	/**
	 * Whether this pouch is a primary autowire candidate.
	 * <p>If this value is {@code true} for exactly one pouch among multiple
	 * matching candidates, it will serve as a tie-breaker.
	 */
  Primary: boolean;
	/**
	 * Specify the factory pouch to use, if any.
	 * This the name of the pouch to call the specified factory method on.
	 * @see #setFactoryMethodName
	 */
  FactoryPouchName: PropertyType;
	/**
	 * Specify a factory method, if any. This method will be invoked with
	 * constructor arguments, or with no arguments if none are specified.
	 * The method will be invoked on the specified factory pouch, if any,
	 * or otherwise as a static method on the local pouch class.
	 * @see #setFactoryPouchName
	 * @see #setPouchClassName
	 */
  FactoryMethodName: PropertyType;
	/**
	 * Return the constructor argument values for this pouch.
	 * <p>The returned instance can be modified during pouch factory post-processing.
	 * @return the ConstructorArgumentValues object (never {@code null})
	 */
  getConstructorArgumentValues(): ConstructorArgumentValues;
	/**
	 * Return the property values to be applied to a new instance of the pouch.
	 * <p>The returned instance can be modified during pouch factory post-processing.
	 * @return the MutablePropertyValues object (never {@code null})
	 */
  getPropertyValues(): MutablePropertyValues;
	/**
	 * Return whether this a <b>Singleton</b>, with a single, shared instance
	 * returned on all calls.
	 * @see #SCOPE_SINGLETON
	 */
	isSingleton(): boolean;

	/**
	 * Return whether this a <b>Prototype</b>, with an independent instance
	 * returned for each call.
	 * @see #SCOPE_PROTOTYPE
	 */
	isPrototype(): boolean;

	/**
	 * Return whether this pouch is "abstract", that is, not meant to be instantiated.
	 */
	isAbstract(): boolean;

	// Read-only attributes

	/**
	 * Return a human-readable description of this pouch definition.
	 */
	getDescription(): string;

	/**
	 * Return a description of the resource that this pouch definition
	 * came from (for the purpose of showing context in case of errors).
	 */
	getResourceDescription(): string;

	/**
	 * Return the originating PouchDefinition, or {@code null} if none.
	 * Allows for retrieving the decorated pouch definition, if any.
	 * <p>Note that this method returns the immediate originator. Iterate through the
	 * originator chain to find the original PouchDefinition as defined by the user.
	 */
	getOriginatingPouchDefinition(): PouchDefinition;
}

export default PouchDefinition;

